use std::sync::Arc;

use specs::{Component, Join, System, VecStorage, Write, WriteStorage};

use crate::canvas::canvas_frame::CanvasFrame;
use crate::canvas::compu_frame::CompuFrame;
use crate::canvas::managed::handles::{CanvasImageHandle, CanvasTextureHandle, CompuBufferHandle, CompuKernelHandle};
use crate::PersistentState;
use crate::render_system::{Geometry, Images, Position, Textures};
use crate::util::vertex::{ImageVertex3D, TextureVertex3D, VertexTypeContainer};
use crate::vkprocessor::VkProcessor;

pub struct Compu {
    pub kernels: Vec<Arc<CompuKernelHandle>>,
    pub buffers: Vec<Arc<CompuBufferHandle>>,
}

impl Component for Compu {
    type Storage = VecStorage<Self>;
}

pub struct CompuSystem;

impl<'a> System<'a> for CompuSystem {
    type SystemData = (
        WriteStorage<'a, Compu>,
        WriteStorage<'a, Images>,
        Write<'a, PersistentState>,     // delta_time, window size, etc.
        Write<'a, VkProcessor>,         // Renderer
    );

    fn run(&mut self, (
        mut compu_list,
        mut image_list,
        mut state,
        mut vk_processor
    ): Self::SystemData) {
        state.compu_frame = CompuFrame::new(state.window_size);

        // compu_frame.add_with_image_swap(compute_buffer.clone(), compute_kernel.clone(), &compu_sprite1);
        // compu_frame.add(compute_buffer.clone(), compute_kernel.clone());

        for (compute_item, image) in (&mut compu_list, &mut image_list).join() {
            state.compu_frame.add_with_image_swap(
                compute_item.buffers.get(0).unwrap().clone(),
                compute_item.kernels.get(0).unwrap().clone(),
                image,
            );
        }

        for (compute_item) in (&mut compu_list).join() {
            state.compu_frame.add(
                compute_item.buffers.get(0).unwrap().clone(),
                compute_item.kernels.get(0).unwrap().clone(),
            );
        }
    }
}